#
# Makefile for the single_top_analysis program
# 
# to build a single_top_analysis exectuable, simply type
# > gmake
#
# to clean up and delete all object and binary files and all temporary files, type
# > gmake clean
#
# Author: Reinhard Schwienhorst
# date: 10/2006
#
# Modifications:
#
####################################################################################
# compiler settings:
CPP=gcc
LD=gcc
# optimize:
SPECIALFLAGS= -O3 
# debug:
#SPECIALFLAGS= -g
ROOTCFLAGS=$(shell root-config --cflags)
ROOTLIBS=$(shell root-config --libs)

INCDIR= single_top_analysis
SRCDIR= src
BINSRCDIR= bin

CFLAGS = $(SPECIALFLAGS) -I- -I. -I${INCDIR} -I${SRCDIR}
LFLAGS = $(SPECIALFLAGS)  /usr/lib/gcc/i486-linux-gnu/4.0.3/libstdc++.a

RCXX=$(CFLAGS) $(ROOTCFLAGS)
RLXX=$(LFLAGS) $(ROOTLIBS)


###################################################


###################################################
# all source files that don't need a dictionary and have a header file
CPPSOURCES= analysis_main HistoCut BaseCut Particles EV0 CutListProcessor myTH1F myTH2F \
utils Particle Cut_Basic Cut_Charge Histogramming_njets Electron CollectionTree Truth0



# sources that don't have a header file
BINSOURCES= test_analysis_x

# sources that need a dictionary
DICTSOURCES=analysis_main HistoCut BaseCut Particles Particle Electron

INC= $(CPPSOURCES:%=${INCDIR}/%.hpp)

SRC= $(CPPSOURCES:%=${SRCDIR}/%.cpp) $(BINSOURCES:%=${BINSRCDIR}/%.cpp) $(DICTSOURCES:%=${SRCDIR}/%dict.cpp)

OBJ= $(CPPSOURCES:%=obj/%.o) $(DICTSOURCES:%=obj/%dict.o) 

#OBJ= $(CPPSOURCES:${SRCDIR}/%.cpp=obj/%.o) $(BINSOURCES:${SRCDIR}/%.cpp=obj/%.o) $(DICTSOURCES:${SRCDIR}/%.cpp=obj/%.o) 

# dependency to build the final binary
all: bin

bin: test_analysis_x

test: $(OBJ)

# build rule for the dictionary files
obj/%dict.o : ${INCDIR}/%.hpp
	@mkdir -p obj
	@mkdir -p tmp
	rootcint -f tmp/$(@:obj/%.o=%.cpp) -c -I${INCDIR}  $< 
	$(CPP) $(RCXX) -Itmp -c tmp/$(@:obj/%.o=%.cpp) -o $@



# build rule for object files
obj/%.o : ${SRCDIR}/%.cpp
	@mkdir -p obj
	$(CPP) $(RCXX) -c $< -o $@

# build rule for the binary
test_analysis_x: $(OBJ) $(INC) bin/test_analysis_x.cpp
	$(CPP) $(RCXX) -c bin/test_analysis_x.cpp -o obj/test_analysis_x.o
	$(LD) $(RLXX) -o test_analysis_x $(OBJ) $(RLXX) obj/test_analysis_x.o


#
# build rule to clean up everything.
clean:
	rm -rf ti_files
	rm -f */*~ $(OBJ) obj/*
	rm -f tmp/*
	rm -f test_analysis_x
	rm -f core
	rm -f */*dict.cpp
	rm -f G__auto*inkDef.h

