///////////////////////////////////////////////////////////////////////////
// header file: Electron
// 
// Description: 
//  store information about final state electrons
//
// Author: Reinhard Schwienhorst
// Date: 10/19/2006
//
// Modifications:
//
//
////////////////////////////////////////////////////////////////////////////
#ifndef electron_h
#define electron_h

#include "single_top_analysis/Particle.hpp"
#include "single_top_analysis/EventTree.hpp"
//

class Electron: public Particle
{
 public:
  // default constructor
  Electron();
  // default constructor
  Electron(const Electron& other);

  // constructor with existing particle
  Electron(const Particle& other);

  //destructor
  ~Electron();

  // clear: set all contents to their defaults
  inline void Clear() { Particle::Clear(); _EoverP=0.; };

  // Fill the electron from an EventTree
  void Fill(EventTree *evtr,int iE);

  // Setters
  // Set E over P for this electron
  void SetEoverP(const double& eop) { _EoverP=eop; };

  // Getters
  double GetEoverP() const { return _EoverP; };
  double EoverP() const { return _EoverP; };


  // overloaded operators:
  // +=
  Electron& operator+=(const Electron& other);
  // addition:
  Electron operator+(const Electron& other);
  // overloaded assignment operator:
  Electron& operator=(const Particle& other);
  // overloaded assignment operator:
  Electron& operator=(const Electron& other);
  // overloaded assignment operator:
  Electron& operator=(Electron& other);
    

 private:
  double _EoverP;   // calo energy of the electron divided by track momentum

  ////////////////////////////////////////
  // needed to run with Root:
  ClassDef(Electron,0)
};


#endif
