//////////////////////////////////////////////////////////////////////////////
//
// helper functions for NLO analysis
//
// Author: Reinhard Schwienhorst
//
// Date: 4/21/04
//
// Modifications:
//
// 9/2004, RS: - split neutrino pz calculation into two parts,
//             - add method to use top mass constraint 
//
/////////////////////////////////////////////////////////////////////////////////
#include <TLorentzVector.h>

#include <math.h>

#include <vector>
//
// function to convert Physics eta into detector eta
// R is the distance to the beamline.
// input: etain, zvertex (in cm).
double etaPhysics2Detector(const double& etain,const double& zvertex,const double& R=72.0);
// input: etain, zvertex (in cm).
double etaDetector2Physics(const double& etain,const double& zvertex,const double& R=72.0);

// find the neutrino PZ
// from top_tree
// Note that if the input scf is Set to a positive number then a scale factor is applied.
//  This means we only apply and return the scale factor if scf is >=0.
double Neutrino_Pz( double nu_px, double nu_py, 
		    double l_px, double l_py, double l_pz, double l_e, 
		    double& scf,double nu_pz_true=0.);

//  Also provide a version that computes neutrino_pz from a W and top mass constraint
double Neutrino_Pz_Top( double nu_px, double nu_py, 
			double l_px, double l_py, double l_pz, double l_e, 
			double b_px, double b_py, double b_pz, double b_e, 
			double nu_pz_true=0.);

//  This means we only apply and return the scale factor if scf is >=0.
void Neutrino_Pz_Base( double nu_px, double nu_py,                        // neutrino 
		       double l_px, double l_py, double l_pz, double l_e, // lepton
		       double& scf,double &solution1, double &solution2); // return variables

//
// find the "best jet", which is the one that gives a top mass
// closest to 175GeV.
int findBestJet(int njets,TLorentzVector* jets,TLorentzVector W,double tarGetmass=175.);

// find the "best jet", which is the one that gives a top mass
// closest to 175GeV, and at the same time find the ideal W.
// That is, combine the top mass constraint and the W mass constraint
int findBestJetW(int njets,TLorentzVector* jets,TLorentzVector ele,TLorentzVector nu,double tarGetmass=175.);
