///////////////////////////////////////////////////////////////////////////////
//
// Histogramming_TopologicalVars
// --------------
//
// Authors: Reinhard Schwienhorst
//         
//
// Date: April 8, 2005
//
// Modified:
// 
//
///////////////////////////////////////////////////////////////////////////////

#include "single_top_analysis/Histogramming_TopologicalVars.hpp"
#include<iostream>

// for the NN output filters:
#include "nnFilter_e_tb_wbb.cpp"
#include "nnFilter_e_tb_lepjets.cpp"
#include "nnFilter_e_tqb_wbb.cpp"
#include "nnFilter_e_tqb_lepjets.cpp"

using namespace std;


//----------------
// constructor
//----------------
Histogramming_TopologicalVars::Histogramming_TopologicalVars(Particles *obj,bool write,TString name,bool doSpin):
  _writeSkim(write),_skimFileName(name),_doTopSpin(doSpin)
{
  SetParticleObj(obj);
  
  // open up the output file if so desired
  if(_writeSkim) {
    cout<<"<Histogramming_TopologicalVars>: Opening topological variables skim file "<<_skimFileName.Data()<<endl;
    _skimFile = new TFile(_skimFileName.Data(),"reCreate"); 
    _skimFile->cd();
    _skimTree = new TTree("TopTree", "My Top Tree");
    
    // now add branches to the tree
    // start with those variables that we are using in the current NNs
    //------------------------------------------------------
    // Event Weight
    //------------------------------------------------------
    _skimTree->Branch("EventWeight", &_EventWeight, "_EventWeight/F");
    _skimTree->Branch("RunNumber", &_RunNumber, "_RunNumber/I");
    _skimTree->Branch("EventNumber", &_EventNumber, "_EventNumber/I");
     
    //------------------------------------------------------
    // System Mass Histograms
    //------------------------------------------------------
    _skimTree->Branch("InvariantMass_AllJets", &_InvariantMass_AllJets, "_InvariantMass_AllJets/F");
    _skimTree->Branch("InvariantMass_Jet1Jet2LeptonMET", &_InvariantMass_Jet1Jet2LeptonMET, "_InvariantMass_Jet1Jet2LeptonMET/F");
    _skimTree->Branch("InvariantMass_AllJetsLeptonMET", &_InvariantMass_AllJetsLeptonMET, "_InvariantMass_AllJetsLeptonMET/F");
    _skimTree->Branch("InvariantMass_LeptonMETBestJet", &_InvariantMass_LeptonMETBestJet, "_InvariantMass_LeptonMETBestJet/F");
    _skimTree->Branch("InvariantMass_AllJets_MinusBestJet", &_InvariantMass_AllJets_MinusBestJet, "_InvariantMass_AllJets_MinusBestJet/F");
    _skimTree->Branch("InvariantMass_AllJets_MinusBTaggedJet", &_InvariantMass_AllJets_MinusBTaggedJet, "_InvariantMass_AllJets_MinusBTaggedJet/F");
    _skimTree->Branch("InvariantMass_Jet1Jet2", &_InvariantMass_Jet1Jet2, "_InvariantMass_Jet1Jet2/F");
    _skimTree->Branch("TransverseMass_Jet1Jet2", &_TransverseMass_Jet1Jet2, "_TransverseMass_Jet1Jet2/F");
    _skimTree->Branch("TransverseMass_Jet1Jet2_LEP", &_TransverseMass_Jet1Jet2_LEP, "_TransverseMass_Jet1Jet2_LEP/F");
    _skimTree->Branch("Shat", &_Shat, "_Shat/F");
     
    //------------------------------------------------------
    // H Histograms
    //------------------------------------------------------
    _skimTree->Branch("H_Jet1Jet2LeptonMET", &_H_Jet1Jet2LeptonMET, "_H_Jet1Jet2LeptonMET/F");
    _skimTree->Branch("H_Jet1Jet2LeptonMET_MinusBJet", &_H_Jet1Jet2LeptonMET_MinusBJet, "_H_Jet1Jet2LeptonMET_MinusBJet/F");
    _skimTree->Branch("H_AllJetsLeptonMET", &_H_AllJetsLeptonMET, "_H_AllJetsLeptonMET/F");
    _skimTree->Branch("H_AllJets", &_H_AllJets, "_H_AllJets/F");
    _skimTree->Branch("H_AllJets_MinusBestJet", &_H_AllJets_MinusBestJet, "_H_AllJets_MinusBestJet/F");
    _skimTree->Branch("H_AllJets_MinusBTaggedJet", &_H_AllJets_MinusBTaggedJet, "_H_AllJets_MinusBTaggedJet/F");
    _skimTree->Branch("H_Jet1Jet2", &_H_Jet1Jet2, "_H_Jet1Jet2/F");
     
    //------------------------------------------------------
    // Ht Histograms
    //------------------------------------------------------
    _skimTree->Branch("HT_Jet1Jet2LeptonMET", &_HT_Jet1Jet2LeptonMET, "_HT_Jet1Jet2LeptonMET/F");
    _skimTree->Branch("HT_Jet1Jet2LeptonMET_MinusBJet", &_HT_Jet1Jet2LeptonMET_MinusBJet, "_HT_Jet1Jet2LeptonMET_MinusBJet/F");
    _skimTree->Branch("HT_AllJetsLeptonMET", &_HT_AllJetsLeptonMET, "_HT_AllJetsLeptonMET/F");
    _skimTree->Branch("HT_AllJets", &_HT_AllJets, "_HT_AllJets/F");
    _skimTree->Branch("HT_AllJets_MinusBestJet", &_HT_AllJets_MinusBestJet, "_HT_AllJets_MinusBestJet/F");
    _skimTree->Branch("HT_AllJets_MinusBTaggedJet", &_HT_AllJets_MinusBTaggedJet, "_HT_AllJets_MinusBTaggedJet/F");
    _skimTree->Branch("HT_Jet1Jet2", &_HT_Jet1Jet2, "_HT_Jet1Jet2/F");
    _skimTree->Branch("Jet3Pt5Jet4Pt", &_Jet3Pt5Jet4Pt, "_Jet3Pt5Jet4Pt/F");
    _skimTree->Branch("Jet1Pt4MET", &_Jet1Pt4MET, "_Jet1Pt4MET/F");
     
     
    //------------------------------------------------------
    // Jet Histograms
    //------------------------------------------------------
    _skimTree->Branch("Jet1Pt", &_Jet1Pt, "_Jet1Pt/F");
    _skimTree->Branch("Jet2Pt", &_Jet2Pt, "_Jet2Pt/F");
    _skimTree->Branch("Jet3Pt", &_Jet3Pt, "_Jet3Pt/F");
    _skimTree->Branch("Jet4Pt", &_Jet4Pt, "_Jet4Pt/F");
    _skimTree->Branch("Jet1Pt_NotBest", &_Jet1Pt_NotBest, "_Jet1Pt_NotBest/F");
    _skimTree->Branch("Jet2Pt_NotBest", &_Jet2Pt_NotBest, "_Jet2Pt_NotBest/F");
    _skimTree->Branch("Pt_Jet1Jet2", &_Pt_Jet1Jet2, "_Pt_Jet1Jet2/F");
    _skimTree->Branch("Pt_AllJets_MinusBestJet", &_Pt_AllJets_MinusBestJet, "_Pt_AllJets_MinusBestJet/F");
    _skimTree->Branch("Pt_AllJets_MinusBTaggedJet", &_Pt_AllJets_MinusBTaggedJet, "_Pt_AllJets_MinusBTaggedJet/F");
    _skimTree->Branch("DeltaPt_Jet1Jet2", &_DeltaPt_Jet1Jet2, "_DeltaPt_Jet1Jet2/F");
    _skimTree->Branch("NJets", &_NJets, "_NJets/I");
    _skimTree->Branch("NGoodJets", &_NGoodJets, "_NGoodJets/I");
    _skimTree->Branch("NTaggedJets", &_NTaggedJets, "_NTaggedJets/I");
    _skimTree->Branch("NUntaggedJets", &_NUntaggedJets, "_NUntaggedJets/I");
    _skimTree->Branch("BestJetPt", &_BestJetPt, "_BestJetPt/F");
    _skimTree->Branch("DeltaRJet1Jet2", &_DeltaRJet1Jet2, "_DeltaRJet1Jet2/F");
    _skimTree->Branch("DeltaPhiLeadingJetLepton", &_DeltaPhiLeadingJetLepton, "_DeltaPhiLeadingJetLepton/F");
    _skimTree->Branch("DeltaPhiLeadingJetMissingEt", &_DeltaPhiLeadingJetMissingEt, "_DeltaPhiLeadingJetMissingEt/F");
    _skimTree->Branch("DeltaRLeadingJetLepton", &_DeltaRLeadingJetLepton, "_DeltaRLeadingJetLepton/F");
     
     
    //------------------------------------------------------
    // Missing Et Histograms
    //------------------------------------------------------
    _skimTree->Branch("METPt", &_METPt, "_METPt/F");
     
    //------------------------------------------------------
    // W Histograms
    //------------------------------------------------------
    _skimTree->Branch("WTransverseMass", &_WTransverseMass, "_WTransverseMass/F");
    _skimTree->Branch("WTransverseMass_LEP", &_WTransverseMass_LEP, "_WTransverseMass_LEP/F");
    _skimTree->Branch("WTransverseMassPrime", &_WTransverseMassPrime, "_WTransverseMassPrime/F");
    _skimTree->Branch("WPt", &_WPt, "_WPt/F");
    _skimTree->Branch("WCharge", &_WCharge, "_WCharge/I");
    _skimTree->Branch("DeltaPhi", &_DeltaPhi, "_DeltaPhi/F");
     
    //------------------------------------------------------
    // Top Histograms
    //------------------------------------------------------
    _skimTree->Branch("BestTopMass", &_BestTopMass, "_BestTopMass/F");
    _skimTree->Branch("BTaggedTopMass", &_BTaggedTopMass, "_BTaggedTopMass/F");
    _skimTree->Branch("DeltaBestTopMass", &_DeltaBestTopMass, "_DeltaBestTopMass/F");
    _skimTree->Branch("DeltaBTaggedTopMass", &_DeltaBTaggedTopMass, "_DeltaBTaggedTopMass/F");
    _skimTree->Branch("LeadingJetTopMass", &_LeadingJetTopMass, "_LeadingJetTopMass/F");
     
    //------------------------------------------------------
    // Lepton Histograms
    //------------------------------------------------------
    _skimTree->Branch("LeptonPt", &_LeptonPt, "_LeptonPt/F");
    _skimTree->Branch("LeptonEta", &_LeptonEta, "_LeptonEta/F");
    _skimTree->Branch("LeptonPhi", &_LeptonPhi, "_LeptonPhi/F");
    _skimTree->Branch("QTimesEtaLepton", &_QTimesEtaLepton, "_QTimesEtaLepton/F");
     
    //------------------------------------------------------
    // UnTagged Jet Histograms
    //------------------------------------------------------
    _skimTree->Branch("LeadingUntaggedJetPt", &_LeadingUntaggedJetPt, "_LeadingUntaggedJetPt/F");
    _skimTree->Branch("SecondUntaggedJetPt", &_SecondUntaggedJetPt, "_SecondUntaggedJetPt/F");
    _skimTree->Branch("QTimesEta", &_QTimesEta, "_QTimesEta/F");
    _skimTree->Branch("DeltaPt_tag_untag", &_DeltaPt_tag_untag, "_DeltaPt_tag_untag/F");
     
    //------------------------------------------------------
    // b-Tagged Jet Histograms
    //------------------------------------------------------
    _skimTree->Branch("LeadingBTaggedJetPt", &_LeadingBTaggedJetPt, "_LeadingBTaggedJetPt/F");
    _skimTree->Branch("SecondBTaggedJetPt", &_SecondBTaggedJetPt, "_SecondBTaggedJetPt/F");
     
    //------------------------------------------------------
    // Angular Variable Histograms
    //------------------------------------------------------
    _skimTree->Branch("Cos_UntaggedJetLepton_BTaggedTop", &_Cos_UntaggedJetLepton_BTaggedTop, "_Cos_UntaggedJetLepton_BTaggedTop/F");
    _skimTree->Branch("Cos_Jet1Lepton_LeadingTop", &_Cos_Jet1Lepton_LeadingTop, "_Cos_Jet1Lepton_LeadingTop/F");
    _skimTree->Branch("Cos_Jet2Lepton_LeadingTop", &_Cos_Jet2Lepton_LeadingTop, "_Cos_Jet2Lepton_LeadingTop/F");
    _skimTree->Branch("Cos_Jet3Lepton_LeadingTop", &_Cos_Jet3Lepton_LeadingTop, "_Cos_Jet3Lepton_LeadingTop/F");
    _skimTree->Branch("Cos_Jet3Lepton_BestTop", &_Cos_Jet3Lepton_BestTop, "_Cos_Jet3Lepton_BestTop/F");
    _skimTree->Branch("Cos_Jet1Lepton_Lab", &_Cos_Jet1Lepton_Lab, "_Cos_Jet1Lepton_Lab/F");
    _skimTree->Branch("Cos_BTaggedJetLepton_Lab", &_Cos_BTaggedJetLepton_Lab, "_Cos_BTaggedJetLepton_Lab/F");
    _skimTree->Branch("Cos_BestJetLepton_Lab", &_Cos_BestJetLepton_Lab, "_Cos_BestJetLepton_Lab/F");
    _skimTree->Branch("Cos_Jet2Lepton_Lab", &_Cos_Jet2Lepton_Lab, "_Cos_Jet2Lepton_Lab/F");
    _skimTree->Branch("Cos_Jet1AllJets_AllJets", &_Cos_Jet1AllJets_AllJets, "_Cos_Jet1AllJets_AllJets/F");
    _skimTree->Branch("Cos_BTaggedJetAllJets_AllJets", &_Cos_BTaggedJetAllJets_AllJets, "_Cos_BTaggedJetAllJets_AllJets/F");
    _skimTree->Branch("Cos_UntaggedJetAllJets_AllJets", &_Cos_UntaggedJetAllJets_AllJets, "_Cos_UntaggedJetAllJets_AllJets/F");
    _skimTree->Branch("Cos_NotBestJetAllJets_AllJets", &_Cos_NotBestJetAllJets_AllJets, "_Cos_NotBestJetAllJets_AllJets/F");
    _skimTree->Branch("Cos_Jet2AllJets_AllJets", &_Cos_Jet2AllJets_AllJets, "_Cos_Jet2AllJets_AllJets/F");
    _skimTree->Branch("Cos_Jet1ZAxis_LeadingTop", &_Cos_Jet1ZAxis_LeadingTop, "_Cos_Jet1ZAxis_LeadingTop/F");
    _skimTree->Branch("Cos_Jet1ZAxis_Lab", &_Cos_Jet1ZAxis_Lab, "_Cos_Jet1ZAxis_Lab/F");
    _skimTree->Branch("Cos_Jet2ZAxis_Lab", &_Cos_Jet2ZAxis_Lab, "_Cos_Jet2ZAxis_Lab/F");
    _skimTree->Branch("Cos_LeptonQZ_BestTop", &_Cos_LeptonQZ_BestTop, "_Cos_LeptonQZ_BestTop/F");
    _skimTree->Branch("Cos_LeptonZpl_BestTop", &_Cos_LeptonZpl_BestTop, "_Cos_LeptonZpl_BestTop/F");
    _skimTree->Branch("Cos_LeptonZmn_BestTop", &_Cos_LeptonZmn_BestTop, "_Cos_LeptonZmn_BestTop/F");
    _skimTree->Branch("Cos_LeptonZpl_LeadingTop", &_Cos_LeptonZpl_LeadingTop, "_Cos_LeptonZpl_LeadingTop/F");
    _skimTree->Branch("Cos_LeptonZmn_LeadingTop", &_Cos_LeptonZmn_LeadingTop, "_Cos_LeptonZmn_LeadingTop/F");
    // also Create branches for the NN output
    _skimTree->Branch("NN_e_tb_wbb",      &_NN_e_tb_wbb,      "_NN_e_tb_wbb/F"     );
    _skimTree->Branch("NN_e_tb_lepjets",  &_NN_e_tb_lepjets,  "_NN_e_tb_lepjets/F" );
    _skimTree->Branch("NN_e_tqb_wbb",     &_NN_e_tqb_wbb,     "_NN_e_tqb_wbb/F"    );
    _skimTree->Branch("NN_e_tqb_lepjets", &_NN_e_tqb_lepjets, "_NN_e_tqb_lepjets/F");
   
  } // end of: if writing skim tree  
     

}

//----------------
// destructor
//----------------
Histogramming_TopologicalVars::~Histogramming_TopologicalVars()
{
  
  if(_writeSkim) {
    cout<<"<Histogramming_TopologicalVars>: closing skim file "<<_skimFileName.Data()<<endl;
    _skimFile->Write();
    _skimFile->Close();
    delete _skimFile;
    _skimFile=0;
  }
}

//----------------
// Book
//----------------
void Histogramming_TopologicalVars::BookHistos(){
  

  // event-wide information
  _hHT =  DeclareTH1F("HT","total transverse energy H_{T}",200,0.,600.);


  _hInvariantMass_AllJets=DeclareTH1F("InvariantMass_AllJets","M(all jets)",200,0.,400.);
  _hInvariantMass_Jet1Jet2LeptonMET=DeclareTH1F("InvariantMass_Jet1Jet2LeptonMET","M(jet1, jet2, lepton, MET)",200,0.,400.);
  _hInvariantMass_AllJetsLeptonMET=DeclareTH1F("InvariantMass_AllJetsLeptonMET","M(all jets, lepton, MET)",200,0.,400.);
  _hInvariantMass_AllJets_MinusBestJet=DeclareTH1F("InvariantMass_AllJets_MinusBestJet","M(all jets minus best jet)",200,0.,400.);
  _hInvariantMass_AllJets_MinusBTaggedJet=DeclareTH1F("InvariantMass_AllJets_MinusBTaggedJet","M(all jets minus b-tagged jet)",200,0.,400.);
  _hInvariantMass_Jet1Jet2=DeclareTH1F("InvariantMass_Jet1Jet2","M(jet1, jet2)",200,0.,400.);

  _hTransverseMass_Jet1Jet2=DeclareTH1F("TransverseMass_Jet1Jet2","M_{T}(jet1, jet2)",150,0.,300.);

  _hH_Jet1Jet2LeptonMET=DeclareTH1F("H_Jet1Jet2LeptonMET","H(jet1, jet2, lepton, MET)",200,0.,600.);
  _hH_AllJetsLeptonMET=DeclareTH1F("H_AllJetsLeptonMET","H(all jets, lepton, MET)",200,0.,600.);
  _hH_AllJets=DeclareTH1F("H_AllJets","H(all jets)",200,0.,600.);
  _hH_AllJets_MinusBestJet=DeclareTH1F("H_AllJets_MinusBestJet","H(all jets minus best jet)",200,0.,400.);
  _hH_AllJets_MinusBTaggedJet=DeclareTH1F("H_AllJets_MinusBTaggedJet","H(all jets minus b-tagged jet)",200,0.,400.);
  _hH_Jet1Jet2=DeclareTH1F("H_Jet1Jet2","H(jet1, jet2)",200,0.,400.);

  _hShat=DeclareTH1F("Shat","Shat",200,0.,1200.);

  _hHT_Jet1Jet2LeptonMET=DeclareTH1F("HT_Jet1Jet2LeptonMET","HT(jet1, jet2, lepton, MET)",200,0.,600.);
  _hHT_AllJetsLeptonMET=DeclareTH1F("HT_AllJetsLeptonMET","HT(all jets, lepton, MET)",200,0.,600.);
  _hHT_AllJets=DeclareTH1F("HT_AllJets","HT(all jets)",200,0.,600.);
  _hHT_AllJets_MinusBestJet=DeclareTH1F("HT_AllJets_MinusBestJet","HT(all jets minus best jet)",200,0.,600.);
  _hHT_AllJets_MinusBTaggedJet=DeclareTH1F("HT_AllJets_MinusBTaggedJet","HT(all jets minus b-tagged jet)",200,0.,600.);
  _hHT_Jet1Jet2=DeclareTH1F("HT_Jet1Jet2","HT(jet1, jet2)",200,0.,400.);

  _hJet1Pt_NotBest=DeclareTH1F("Jet1Pt_NotBest","P_{T} (jet1 not best)",200,0.,200.);
  _hJet2Pt_NotBest=DeclareTH1F("Jet2Pt_NotBest","P_{T} (jet2 not best)",200,0.,200.);
  
  _hPt_Jet1Jet2=DeclareTH1F("Pt_Jet1Jet2","P_{T} (jet1, jet2)",200,0.,200.);
  _hPt_AllJets_MinusBestJet=DeclareTH1F("Pt_AllJets_MinusBestJet","P_{T} (all jets minus best jet",200,0.,400.);
  _hPt_AllJets_MinusBTaggedJet=DeclareTH1F("Pt_AllJets_MinusBTaggedJet","P_{T} (all jets minus b-tagged jet",200,0.,400.);
  _hDeltaPt_Jet1Jet2=DeclareTH1F("DeltaPt_Jet1Jet2","#DeltaP_{T} (jet1, jet2)",200,0.,100.);
  _hBestJetPt=DeclareTH1F("BestJetPt","P_{T} (best jet)",200,0.,200.);

  _hNJets=DeclareTH1F("NJets","N jets",5,-0.5,4.5);
  _hNGoodJets=DeclareTH1F("NGoodJets","N good jets",5,-0.5,4.5);
  _hNTaggedJets=DeclareTH1F("NTaggedJets","N tagged jets",5,-0.5,4.5);
  _hNUntaggedJets=DeclareTH1F("NUntaggedJets","N untagged jets",5,-0.5,4.5);

  _hDeltaRJet1Jet2=DeclareTH1F("DeltaRJet1Jet2","#DeltaR (jet1, jet2)",100,0.,10.);

  _hWTransverseMass=DeclareTH1F("WTransverseMass","MT(W)",150,0.,150.);
  _hWCharge=DeclareTH1F("WCharge","Charge(W)",3,-1.,1.);

  _hBestTopMass=DeclareTH1F("BestTopMass","M(best top)",200,100.,300.);
  _hBTaggedTopMass=DeclareTH1F("BTaggedTopMass","M(b-tagged top)",250,100.,350.);

  _hLeadingUntaggedJetPt=DeclareTH1F("LeadingUntaggedJetPt","PT(leading untagged jet)",200,0.,200.);
  _hSecondUntaggedJetPt=DeclareTH1F("SecondUntaggedJetPt","PT(second untagged jet)",200,0.,200.);

  _hQTimesEta=DeclareTH1F("QTimesEta","Q(lepton) #times #eta(leading untagged jet)",100,-5.,5.);
  _hDeltaPt_tag_untag=DeclareTH1F("DeltaPt_tag_untag","#DeltaP_{T} (jet_{tagged}, jet_{untagged})",200,-50.,150.);

  _hLeadingBTaggedJetPt=DeclareTH1F("LeadingBTaggedJetPt","PT(leading b-tagged jet)",200,0.,200.);
  _hSecondBTaggedJetPt=DeclareTH1F("SecondBTaggedJetPt","PT(second b-tagged jet)",200,0.,200.);

  _hCos_UntaggedJetLepton_BTaggedTop=DeclareTH1F("Cos_UntaggedJetLepton_BTaggedTop","cosine(untagged jet, lepton)_{b-tagged top}",100,-1.,1.);
  _hCos_Jet1AllJets_AllJets=DeclareTH1F("Cos_Jet1AllJets_AllJets","cosine(jet1, all jets)_{all jets}",100,-1.,1.);
  _hCos_BTaggedJetAllJets_AllJets=DeclareTH1F("Cos_BTaggedJetAllJets_AllJets","cosine(b-tagged jet, all jets)_{all jets}",100,-1.,1.);
  _hCos_UntaggedJetAllJets_AllJets=DeclareTH1F("Cos_UntaggedJetAllJets_AllJets","cosine(untagged jet, all jets)_{all jets}",100,-1.,1.);
  _hCos_NotBestJetAllJets_AllJets=DeclareTH1F("Cos_NotBestJetAllJets_AllJets","cosine(not best jet, all jets)_{all jets}",100,-1.,1.);
  _hCos_LeptonQZ_BestTop=DeclareTH1F("Cos_LeptonQZ_BestTop","cosine(lepton, Q#timesZ)_{best top}",100,-1.,1.);
  


  _hNN_e_tb_wbb =      DeclareTH1F("NN_e_tb_wbb","NN output, e, tb-Wbb",200,-0.5,1.5);
  _hNN_e_tb_lepjets =  DeclareTH1F("NN_e_tb_lepjets","NN output, e, tb-t#bar{t}",200,-0.5,1.5);
  _hNN_e_tqb_wbb =     DeclareTH1F("NN_e_tqb_wbb","NN output, e, tqb-Wbb",200,-0.5,1.5);
  _hNN_e_tqb_lepjets = DeclareTH1F("NN_e_tqb_lepjets","NN output, e, tqb-t#bar{t}",200,-0.5,1.5);

  _hNN_e_tb_wbb_lepjets = DeclareTH2F("NN_e_tb_wbb_lepjets","NN output, e, tb, wbb vs t#bar{t}",200,-0.5,1.5,200,-0.5,1.5);
  _hNN_e_tqb_wbb_lepjets = DeclareTH2F("NN_e_tqb_wbb_lepjets","NN output, e, tqb, wbb vs t#bar{t}",200,-0.5,1.5,200,-0.5,1.5);

  _hNN_e_tb_tqb_ttcut = DeclareTH2F("NN_e_tb_tqb_ttcut","NN output, e, tb vs tqb, t#bar{t} >0.4",200,-0.5,1.5,200,-0.5,1.5);

  
}

//-------------
// Apply
//-------------
bool Histogramming_TopologicalVars::Apply()
{
  //cout<<"Begin of Histogramming_TopologicalVars::Apply()"<<endl;
  Particles *obj=GetParticleObj();
  
  
  Double_t HT=0.,tempd;
  
  HT+=obj->finalLepton.Pt();
  HT+=obj->finalNeutrino.Pt();
  
  for(int ijet=0;ijet<obj->njets;ijet++) {
    HT+=obj->finalJets[ijet].Pt();
  }
  _hHT->Fill(HT);



  //------------------------------------------------------
  // Event Weight
  //------------------------------------------------------
  _EventWeight = obj->wgt;
  _EventNumber = obj->evtNum;
  _RunNumber = 1;
  
  //------------------------------------------------------
  // System Mass Histograms
  //------------------------------------------------------
  TLorentzVector tlJets(0,0,0,0);
  for(int i=0;i<obj->njets;i++) tlJets += obj->finalJets[i];
  
  _InvariantMass_AllJets = tlJets.M();
  _hInvariantMass_AllJets->Fill(_InvariantMass_AllJets);

  TLorentzVector tlJet12(0,0,0,0);
  if(obj->njets>0) tlJet12 += obj->finalJets[0];
  if(obj->njets>1) tlJet12 += obj->finalJets[1];

  _InvariantMass_Jet1Jet2 =  tlJet12.M();
  _hInvariantMass_Jet1Jet2->Fill(_InvariantMass_Jet1Jet2);

  //------------------
  // the following was taken from the root TLorenzVector
  // class, because root versions before 4.something
  // don't have a member function Et yet.
  Double_t ET2=0.;
  // 
  Double_t pt2 = tlJet12.Perp2();
  if(pt2>0.)
    ET2 = tlJet12.E()*tlJet12.E() * pt2/(pt2+tlJet12.Z()*tlJet12.Z());
  //------------------

  tempd= ET2 - pow(tlJet12.Px(),2) - pow(tlJet12.Py(),2);
  if(tempd>0) tempd=TMath::Sqrt(tempd);
  else tempd=-1.;
  _TransverseMass_Jet1Jet2 = tempd;
  _hTransverseMass_Jet1Jet2->Fill(_TransverseMass_Jet1Jet2);

  TLorentzVector tlJetsminusBest(tlJets);
  tlJetsminusBest -= *(obj->bestJetPtr);
  _InvariantMass_AllJets_MinusBestJet = tlJetsminusBest.M();
  if(_InvariantMass_AllJets_MinusBestJet<0.) _InvariantMass_AllJets_MinusBestJet=0.;
  _hInvariantMass_AllJets_MinusBestJet->Fill(_InvariantMass_AllJets_MinusBestJet);

  _Pt_AllJets_MinusBestJet = tlJetsminusBest.Pt();
  _hPt_AllJets_MinusBestJet->Fill(_Pt_AllJets_MinusBestJet);

  TLorentzVector tlJetsminusTag(tlJets);
  tlJetsminusTag -= *(obj->taggedJetPtr);
  _InvariantMass_AllJets_MinusBTaggedJet = tlJetsminusTag.M();
  _hInvariantMass_AllJets_MinusBTaggedJet->Fill(_InvariantMass_AllJets_MinusBTaggedJet);

  _Pt_AllJets_MinusBTaggedJet = tlJetsminusTag.Pt();
  _hPt_AllJets_MinusBTaggedJet->Fill(_Pt_AllJets_MinusBTaggedJet);

  //_InvariantMass_Jet1Jet2LeptonMET = InvariantMass_Jet1Jet2LeptonMET(obj);
  //_InvariantMass_AllJetsLeptonMET = InvariantMass_AllJetsLeptonMET(obj);
  //_InvariantMass_LeptonMETBestJet = InvariantMass_LeptonMETBestJet(obj);
  //_TransverseMass_Jet1Jet2_LEP = TransverseMass_Jet1Jet2_LEP(obj);
  _Shat = 0.;
  TLorentzVector tlAll=tlJets;
  tlAll+=obj->finalLepton;
  tlAll+=obj->finalNeutrino;
  _Shat=tlAll.M();
  _hShat->Fill(_Shat);

  //------------------------------------------------------
  // H Histograms
  //------------------------------------------------------
  _H_Jet1Jet2 = 0.;
  _HT_Jet1Jet2 = 0.;
  if(obj->njets>0) {
    _H_Jet1Jet2 += obj->finalJets[0].E();
    _HT_Jet1Jet2 += obj->finalJets[0].Pt();
  }
  if(obj->njets>1) {
    _H_Jet1Jet2 += obj->finalJets[1].E();
    _HT_Jet1Jet2 += obj->finalJets[1].Pt();
  }
  _hH_Jet1Jet2->Fill(_H_Jet1Jet2);
  _hHT_Jet1Jet2->Fill(_HT_Jet1Jet2);

  _H_Jet1Jet2LeptonMET = _H_Jet1Jet2 + obj->finalLepton.E() + obj->finalNeutrino.Pt();
  _hH_Jet1Jet2LeptonMET->Fill(_H_Jet1Jet2LeptonMET);

  _H_Jet1Jet2LeptonMET_MinusBJet = _H_Jet1Jet2LeptonMET;
  if(obj->taggedJetPtr!=0) _H_Jet1Jet2LeptonMET_MinusBJet -= obj->taggedJetPtr->E();
  //_hH_Jet1Jet2LeptonMET_MinusBJet->Fill(_H_Jet1Jet2LeptonMET_MinusBJet);

  _H_AllJetsLeptonMET = _H_Jet1Jet2LeptonMET;
  if(obj->njets>2) _H_AllJetsLeptonMET += obj->finalJets[2].E();
  if(obj->njets>3) _H_AllJetsLeptonMET += obj->finalJets[3].E();
  _hH_AllJetsLeptonMET->Fill(_H_AllJetsLeptonMET);
  
  _H_AllJets = 0.;
  _HT_AllJets = 0.;
  for(int i=0;i<obj->njets;i++) {
    _H_AllJets+=obj->finalJets[i].E();
    _HT_AllJets+=obj->finalJets[i].Pt();
  }
  _hH_AllJets->Fill(_H_AllJets);
  _hHT_AllJets->Fill(_HT_AllJets);
  
  _H_AllJets_MinusBestJet = _H_AllJets - obj->bestJetPtr->E();
  _HT_AllJets_MinusBestJet = _HT_AllJets - obj->bestJetPtr->Pt();
  _hH_AllJets_MinusBestJet->Fill(_H_AllJets_MinusBestJet);
  _hHT_AllJets_MinusBestJet->Fill(_HT_AllJets_MinusBestJet);

  _H_AllJets_MinusBTaggedJet = _H_AllJets - obj->taggedJetPtr->E();
  _HT_AllJets_MinusBTaggedJet = _HT_AllJets - obj->taggedJetPtr->Pt();
  _hH_AllJets_MinusBTaggedJet->Fill(_H_AllJets_MinusBTaggedJet);
  _hHT_AllJets_MinusBTaggedJet->Fill(_HT_AllJets_MinusBTaggedJet);
  
  //------------------------------------------------------
  // Ht Histograms
  //------------------------------------------------------
  //_HT_Jet1Jet2LeptonMET = HT_Jet1Jet2LeptonMET(obj);
  //_HT_Jet1Jet2LeptonMET_MinusBJet = HT_Jet1Jet2LeptonMET_MinusBJet(obj);
  //_HT_AllJetsLeptonMET = HT_AllJetsLeptonMET(obj);
  //_Jet3Pt5Jet4Pt = Jet3Pt5Jet4Pt(obj);
  //_Jet1Pt4MET = Jet1Pt4MET(obj);
  
  //------------------------------------------------------
  // Jet Histograms
  //------------------------------------------------------
  _NJets = obj->njets;
  _hNJets->Fill(_NJets);

  _NTaggedJets = 0;
  int ntags=0;
  for(int i=0;i<obj->njets;i++) if(obj->finalJets[i].isTagged()) ntags++;
  _NTaggedJets =ntags;
  _hNTaggedJets->Fill(_NTaggedJets);

  _NUntaggedJets = obj->njets - ntags;
  _hNUntaggedJets->Fill(_NUntaggedJets);

  _NGoodJets = obj->njets;
  _hNGoodJets->Fill(_NGoodJets);
    
  _Jet1Pt = 0.;
  if(obj->njets>0) _Jet1Pt = obj->finalJets[0].Pt();

  _Jet2Pt = 0.;
  if(obj->njets>1) _Jet2Pt = obj->finalJets[1].Pt();

  _Jet3Pt = 0.;
  if(obj->njets>2) _Jet3Pt = obj->finalJets[2].Pt();

  _Jet4Pt = 0.;
  if(obj->njets>3) _Jet4Pt = obj->finalJets[3].Pt();

  _DeltaPt_Jet1Jet2 = _Jet1Pt - _Jet2Pt;
  _hDeltaPt_Jet1Jet2->Fill(_DeltaPt_Jet1Jet2);

  _DeltaRJet1Jet2 = 0.;
  if(obj->njets>1 && obj->finalJets[0].Perp2()>0. && obj->finalJets[1].Perp2()>0.) {
    _DeltaRJet1Jet2=obj->finalJets[0].DeltaR(obj->finalJets[1]);
  }
  _hDeltaRJet1Jet2->Fill(_DeltaRJet1Jet2);
  
  _Jet1Pt_NotBest = -1;
  _Jet2Pt_NotBest = -1;
  bool foundJet1=false;
  for(int i=0;i<obj->njets;i++) {
    if(!foundJet1  &&  &(obj->finalJets[i])!=obj->bestJetPtr) {
      _Jet1Pt_NotBest = obj->finalJets[i].Pt();
      foundJet1=true;
    }
    else if(foundJet1  &&  &(obj->finalJets[i])!=obj->bestJetPtr) {
      _Jet2Pt_NotBest = obj->finalJets[i].Pt();
      break;
    }    
  } // end of loop over jets
  _hJet1Pt_NotBest->Fill(_Jet1Pt_NotBest);
  _hJet2Pt_NotBest->Fill(_Jet2Pt_NotBest);
  

  _Pt_Jet1Jet2 = 0.;
  if(obj->njets>1) _Pt_Jet1Jet2 = (obj->finalJets[0] + obj->finalJets[1]).Pt();
  _hPt_Jet1Jet2->Fill(_Pt_Jet1Jet2);
  
  _BestJetPt = obj->bestJetPtr->Pt();
  _hBestJetPt->Fill(_BestJetPt);

  //_DeltaPhiLeadingJetLepton = DeltaPhi(LeadingJet(obj), GetTheLepton(obj));
  //_DeltaPhiLeadingJetMissingEt = DeltaPhi(LeadingJet(obj), GetTheMissingEt(obj));
  //_DeltaRLeadingJetLepton = DeltaR(LeadingJet(obj), GetTheLepton(obj));
  
  //------------------------------------------------------
  // UnTagged Jet Histograms
  //------------------------------------------------------
  _LeadingUntaggedJetPt = -1;
  if(obj->untaggedPtJetPtr!=0) _LeadingUntaggedJetPt = obj->untaggedPtJetPtr->Pt();
  _hLeadingUntaggedJetPt->Fill(_LeadingUntaggedJetPt);
  
  _SecondUntaggedJetPt = -1;
  foundJet1=false;
  for(int i=0;i<obj->njets;i++) {
    if(!foundJet1  &&  &(obj->finalJets[i])==obj->untaggedPtJetPtr) {
      foundJet1=true;
    }
    else if(foundJet1  &&  !(obj->finalJets[i].isTagged())) {
      _SecondUntaggedJetPt = obj->finalJets[i].Pt();
      break;
    }    
  } // end of loop over jets
  _hSecondUntaggedJetPt->Fill(_SecondUntaggedJetPt);

  _QTimesEta = 0.;
  if(obj->untaggedPtJetPtr!=0)
    if(obj->untaggedPtJetPtr->Perp2()>0.) 
      _QTimesEta = obj->untaggedPtJetPtr->Eta() * obj->finalLepton.charge();
  _hQTimesEta->Fill(_QTimesEta);
  
  //_DeltaPt_tag_untag = DeltaPtTagUntag(obj);
  
  //------------------------------------------------------
  // b-Tagged Jet Histograms
  //------------------------------------------------------
  _LeadingBTaggedJetPt = 0.;
  if(obj->taggedJetPtr!=0) _LeadingBTaggedJetPt = obj->taggedJetPtr->Pt();
  _hLeadingBTaggedJetPt->Fill(_LeadingBTaggedJetPt);
  
  _SecondBTaggedJetPt = -1;
  foundJet1=false;
  for(int i=0;i<obj->njets;i++) {
    if(!foundJet1  &&  &(obj->finalJets[i])==obj->taggedJetPtr) {
      foundJet1=true;
    }
    else if(foundJet1  &&  obj->finalJets[i].isTagged()) {
      _SecondBTaggedJetPt = obj->finalJets[i].Pt();
      break;
    }    
  } // end of loop over jets
  _hSecondBTaggedJetPt->Fill(_SecondBTaggedJetPt);

  
  //------------------------------------------------------
  // Missing Et
  //------------------------------------------------------
  _METPt = obj->finalNeutrino.Pt();
  
  //------------------------------------------------------
  // W Histograms
  //------------------------------------------------------
  _WTransverseMass = TMath::Sqrt(pow(obj->finalW.Pt(),2) - pow(obj->finalW.Px(),2) + pow(obj->finalW.Py(),2));
  _hWTransverseMass->Fill(_WTransverseMass);

  //_WTransverseMass_LEP = obj->finalW.Mt();

  _WCharge = static_cast<int>(obj->finalW.charge());
  _hWCharge->Fill(_WCharge);

  _WPt = obj->finalW.Pt();

  //_DeltaPhi = DeltaPhi(_lepton_type, _lepton_cut_name, obj);
  
  //------------------------------------------------------
  // Top Histograms
  //------------------------------------------------------
  _BestTopMass = obj->finalTop_bestJet.M();
  _hBestTopMass->Fill(_BestTopMass);

  _BTaggedTopMass = obj->finalTop_taggedJet.M();
  _hBTaggedTopMass->Fill(_BTaggedTopMass);

  //_DeltaBestTopMass = DeltaBestTopMass(obj);
  //_DeltaBTaggedTopMass = DeltaBTaggedTopMass(obj);
  _LeadingJetTopMass = obj->finalTop_leadingJet.M();
  //_hLeadingJetTopMass->Fill(_LeadingJetTopMass);

  
  //------------------------------------------------------
  // Lepton Histograms
  //------------------------------------------------------
  _LeptonPt = obj->finalLepton.Pt();

  _LeptonEta = 0.;
  if(_LeptonPt>0.) _LeptonEta = obj->finalLepton.Eta();

  _LeptonPhi = obj->finalLepton.Phi();

  _QTimesEtaLepton = _LeptonEta * obj->finalLepton.charge();
  
  //------------------------------------------------------
  // Angular Variable Histograms
  //------------------------------------------------------
  Double_t tempcos,tempcharge;
  TLorentzVector tlPbar;
  tempcharge=-obj->finalLepton.charge();
  tlPbar.SetPxPyPzE(0.,0.,tempcharge,1.);
  TLorentzVector tempL=obj->finalLepton;
  tempL.Boost(-obj->finalTop_bestJet.BoostVector());
  tlPbar.Boost(-obj->finalTop_bestJet.BoostVector());
  tempcos=tempL.Vect().Dot(tlPbar.Vect());
  tempcos=tempcos / (tlPbar.Vect().Mag()*tempL.Vect().Mag());
  _Cos_LeptonQZ_BestTop = tempcos;
  _hCos_LeptonQZ_BestTop->Fill(_Cos_LeptonQZ_BestTop);
  
  TLorentzVector tempall=tlJets;
  TLorentzVector tempjet=*(obj->nonBestJet12Ptr);
  tempjet.Boost(-tempall.BoostVector());
  tempcos=tempjet.Vect().Dot(tempall.Vect());
  tempcos=tempcos / (tempjet.Vect().Mag()*tempall.Vect().Mag());
  _Cos_NotBestJetAllJets_AllJets = tempcos;
  _hCos_NotBestJetAllJets_AllJets->Fill(_Cos_NotBestJetAllJets_AllJets);

  tempall=tlJets;
  tempjet=*(obj->taggedJetPtr);
  tempjet.Boost(-tempall.BoostVector());
  tempcos=tempjet.Vect().Dot(tempall.Vect());
  tempcos=tempcos / (tempjet.Vect().Mag()*tempall.Vect().Mag());
  _Cos_BTaggedJetAllJets_AllJets = tempcos;
  _hCos_BTaggedJetAllJets_AllJets->Fill(_Cos_BTaggedJetAllJets_AllJets);
  
  _Cos_UntaggedJetLepton_BTaggedTop = 0.;
  if(obj->untaggedPtJetPtr!=0) {
    tempL=obj->finalLepton;
    tempL.Boost(-obj->finalTop_taggedJet.BoostVector());
    tempjet=*(obj->untaggedPtJetPtr);
    tempjet.Boost(-obj->finalTop_taggedJet.BoostVector());
    tempcos=tempL.Vect().Dot(tempjet.Vect());
    tempcos=tempcos / (tempjet.Vect().Mag()*tempL.Vect().Mag());
    _Cos_UntaggedJetLepton_BTaggedTop = tempcos;
  }
  _hCos_UntaggedJetLepton_BTaggedTop->Fill(_Cos_UntaggedJetLepton_BTaggedTop);

  //_Cos_Jet1ZAxis_Lab = CosineJet1QZAxisLabFrame(obj);
  //_Cos_Jet2ZAxis_Lab = CosineJet2QZAxisLabFrame(obj);
  //_Cos_Jet1Lepton_Lab = CosineJet1LeptonLabFrame(obj);
  //_Cos_BTaggedJetLepton_Lab = CosineBTaggedJetLeptonLabFrame(obj);
  //_Cos_BestJetLepton_Lab = CosineBestJetLeptonLabFrame(obj);
  //_Cos_Jet2Lepton_Lab = CosineJet2LeptonLabFrame(obj);
  //_Cos_Jet1Lepton_LeadingTop = CosineJet1LeptonLeadingTopFrame(obj);
  //_Cos_Jet2Lepton_LeadingTop = CosineJet2LeptonLeadingTopFrame(obj);
  //_Cos_Jet3Lepton_LeadingTop = CosineJet3LeptonLeadingTopFrame(obj);
  //_Cos_Jet3Lepton_BestTop = CosineJet3LeptonBestTopFrame(obj);

  tempall=tlJets;
  tempjet=(obj->finalJets[0]);
  tempjet.Boost(-tempall.BoostVector());
  tempcos=tempjet.Vect().Dot(tempall.Vect());
  tempcos=tempcos / (tempjet.Vect().Mag()*tempall.Vect().Mag());
  _Cos_Jet1AllJets_AllJets = tempcos;
  _hCos_Jet1AllJets_AllJets->Fill(_Cos_Jet1AllJets_AllJets);
  
  _Cos_UntaggedJetAllJets_AllJets = 0.;
  if(obj->untaggedPtJetPtr!=0) {
    tempall=tlJets;
    tempjet=*(obj->untaggedPtJetPtr);
    tempjet.Boost(-tempall.BoostVector());
    tempcos=tempjet.Vect().Dot(tempall.Vect());
    tempcos=tempcos / (tempjet.Vect().Mag()*tempall.Vect().Mag());
    _Cos_UntaggedJetAllJets_AllJets = tempcos;
  }
  _hCos_UntaggedJetAllJets_AllJets->Fill(_Cos_UntaggedJetAllJets_AllJets);
  
  //_Cos_Jet2AllJets_AllJets = CosineJet1AllJetsAllJetsFrame(obj);
  //_Cos_Jet1ZAxis_LeadingTop = CosineJet1ZAxisLeadingTopFrame(obj);
  //_Cos_Jet1ZAxis_Lab = CosineJet1QZAxisLabFrame(obj);
  //_Cos_Jet2ZAxis_Lab = CosineJet2QZAxisLabFrame(obj);
  //_Cos_LeptonZpl_BestTop = CosineLeptonZPlusAxisBestTopFrame(obj);
  //_Cos_LeptonZmn_BestTop = CosineLeptonZMinusAxisBestTopFrame(obj);

  //---------------------------------------------------------
  // now calculate NN output values
  //
  Double_t NNinVars[11];
  Double_t NNout;

  // a) tb-wbb filter
  NNinVars[0]=log(_LeadingBTaggedJetPt);
  NNinVars[1]=log(_Shat);
  NNinVars[2]=log(_DeltaRJet1Jet2);
  NNinVars[3]=log(_BTaggedTopMass);
  NNinVars[4]=log(_BestTopMass);
  NNinVars[5]=0.;
  if(_doTopSpin) NNinVars[5]=_Cos_LeptonQZ_BestTop;
  NNinVars[6]=log(_Jet1Pt_NotBest);
  NNinVars[7]=0.;
  if(_Jet2Pt_NotBest>0.) NNinVars[7]=log(_Jet2Pt_NotBest);
  NNinVars[8]=log(_Pt_Jet1Jet2);
  NNinVars[9]=log(_InvariantMass_AllJets);
  NNinVars[10]=log(_TransverseMass_Jet1Jet2);
  // now calculate the output value
  e_tb_wbb::rnnfun(NNinVars,&NNout);
  _NN_e_tb_wbb = NNout;

  // b) tb-lepjets
  NNinVars[0]=log(_LeadingBTaggedJetPt);
  NNinVars[1]=log(_Jet1Pt_NotBest);
  NNinVars[2]=0.;
  if(_Jet2Pt_NotBest>0.) NNinVars[2]=log(_Jet2Pt_NotBest);
  NNinVars[3]=log(_InvariantMass_AllJets);
  NNinVars[4]=log(_InvariantMass_AllJets_MinusBestJet);
  NNinVars[5]=log(_H_AllJets_MinusBestJet);
  NNinVars[6]=log(_HT_AllJets_MinusBestJet);
  NNinVars[7]=log(_H_AllJets_MinusBTaggedJet);
  NNinVars[8]=log(_Pt_AllJets_MinusBTaggedJet);
  NNinVars[9]=log(_BTaggedTopMass);
  NNinVars[10]=_Cos_NotBestJetAllJets_AllJets;
  // now calculate the output value
  e_tb_lepjets::rnnfun(NNinVars,&NNout);
  _NN_e_tb_lepjets = NNout;

  // c) tqb-wbb filter
  NNinVars[0]=log(_LeadingBTaggedJetPt);
  NNinVars[1]=log(_LeadingUntaggedJetPt);
  NNinVars[2]=log(_Pt_Jet1Jet2);
  NNinVars[3]=log(_InvariantMass_AllJets);
  NNinVars[4]=log(_Shat);
  NNinVars[5]=log(_DeltaRJet1Jet2);
  NNinVars[6]=log(_HT_AllJets);
  NNinVars[7]=_Cos_BTaggedJetAllJets_AllJets;
  NNinVars[8]=log(_BTaggedTopMass);
  NNinVars[9]=_QTimesEta;
  NNinVars[10]=0.;
  if(_doTopSpin) NNinVars[10]=_Cos_UntaggedJetLepton_BTaggedTop;
  // now calculate the output value
  e_tqb_wbb::rnnfun(NNinVars,&NNout);
  _NN_e_tqb_wbb = NNout;

  // d) tqb-lepjets
  NNinVars[0]=log(_LeadingUntaggedJetPt);
  NNinVars[1]=0.;
  if(_SecondUntaggedJetPt>0.) NNinVars[1]=log(_SecondUntaggedJetPt);
  NNinVars[2]=log(_InvariantMass_AllJets);
  NNinVars[3]=log(_InvariantMass_AllJets_MinusBTaggedJet);
  NNinVars[4]=log(_H_AllJets_MinusBTaggedJet);
  NNinVars[5]=log(_HT_AllJets_MinusBTaggedJet);
  NNinVars[6]=log(_Pt_AllJets_MinusBTaggedJet);
  NNinVars[7]=log(_Shat);
  NNinVars[8]=log(_BTaggedTopMass);
  NNinVars[9]=_QTimesEta;
  NNinVars[10]=_Cos_BTaggedJetAllJets_AllJets;
  // now calculate the output value
  e_tqb_lepjets::rnnfun(NNinVars,&NNout);
  _NN_e_tqb_lepjets = NNout;

  //
  // and fill all the histograms
  _hNN_e_tb_wbb->Fill(_NN_e_tb_wbb);
  _hNN_e_tb_lepjets->Fill(_NN_e_tb_lepjets);
  _hNN_e_tqb_wbb->Fill(_NN_e_tqb_wbb);
  _hNN_e_tqb_lepjets->Fill(_NN_e_tqb_lepjets);
  //
  _hNN_e_tb_wbb_lepjets->Fill(_NN_e_tb_wbb,_NN_e_tb_lepjets);
  _hNN_e_tqb_wbb_lepjets->Fill(_NN_e_tqb_wbb,_NN_e_tqb_lepjets);
  
  if(_NN_e_tb_lepjets>0.4 && _NN_e_tqb_lepjets>0.4) {
    _hNN_e_tb_tqb_ttcut->Fill(_NN_e_tb_wbb,_NN_e_tqb_wbb);
  }
  //
  // finally, write out the skim
  if ( _writeSkim ) {
    _skimTree->Fill();
  }



  //cout<<"End of Histogramming_TopologicalVars::Apply()"<<endl;
  return true;  
  
}










